// file      : driver.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : not copyrighted - public domain

#include <memory>   // std::auto_ptr
#include <iostream>
#include <ctime>    // time()
#include <cstdlib>  // rand()

#include <boost/date_time/posix_time/posix_time.hpp>

#include <odb/database.hxx>
#include <odb/transaction.hxx>
#include <odb/schema-catalog.hxx>

#include "time.hxx"
#include "database.hxx" // create_database

#include "post.hxx"
#include "post-odb.hxx"

using namespace std;
using namespace odb::core;

using namespace boost::gregorian;
using namespace boost::posix_time;

const char* text =
  "LINQ 2 SQL elapsed 335 ms"
  "LINQ 2 SQL compiled 207 ms"
  "LINQ 2 SQL ExecuteQuery 242 ms"
  "Sams ORM 174 ms"
  "Entity Framework 4.1 elapsed 550ms"
  "Hand coded 164ms";

const unsigned long total_objects = 10000;

void
test (database& db)
{
  post p;

  // Load random post objects.
  //
  transaction t (db.begin ());

  for (unsigned long i (0); i < 500; ++i)
  {
    unsigned long id (rand () % total_objects);
    db.load (id, p);
  }

  t.commit ();
}

void
test1 (database& db)
{
  typedef odb::query<post> query;
  typedef odb::result<post> result;

  post p;

  unsigned long id;
  query q (query::id == query::_ref (id));

  // Load random post objects.
  //
  transaction t (db.begin ());

  for (unsigned long i (0); i < 500; ++i)
  {
    id = rand () % total_objects;

    result r (db.query<post> (q));
    r.begin ().load (p);
  }

  t.commit ();
}

int
main (int argc, char* argv[])
{
  srand (time (0));

  try
  {
    auto_ptr<database> db (create_database (argc, argv));

    // Create the database schema.
    //
    {
      transaction t (db->begin ());
      schema_catalog::create_schema (*db);
      t.commit ();
    }

    // Create a few posts.
    //
    {
      transaction t (db->begin ());

      for (unsigned long i (0); i < total_objects; ++i)
      {
        post p;

        p.id = i;
        p.text = text;
        p.creation_date = second_clock::local_time () - time_duration (i, 0, 0);
        p.last_change_date = second_clock::local_time ();

        p.counter1 = i + 1;
        p.counter2 = i + 2;
        p.counter3 = i + 3;
        p.counter4 = i + 4;
        p.counter5 = i + 5;
        p.counter6 = i + 6;
        p.counter7 = i + 7;
        p.counter8 = i + 8;
        p.counter9 = i + 9;

        db->persist (p);
      }

      t.commit ();
    }

    // Warmup.
    //
    for (unsigned long i (0); i < 100; ++i)
      test (*db);

    os::time start;

    for (unsigned long i (0); i < 400; ++i)
      test (*db);

    os::time end;


    cerr << "400 test runs of 500 iterations completed in "
         << end - start << " sec" << endl;
  }
  catch (const odb::exception& e)
  {
    cerr << e.what () << endl;
    return 1;
  }
}
